@@ -1,9 +1,23 @@
-FRESAScale <- function(data,refFrame=NULL,method=c("Norm","Order","OrderLogit","RankInv"),refMean=NULL,refDisp=NULL,strata=NA)
+FRESAScale <- function(data,refFrame=NULL,method=c("Norm","Order","OrderLogit","RankInv","LRankInv"),refMean=NULL,refDisp=NULL,strata=NA)
 {
 	if (is.null(refFrame))
 	{
 		refFrame <- as.data.frame(data);
 	}
+	
+	if (inherits(refFrame,"character"))
+	{
+		refFrame <- refFrame[refFrame %in% rownames(data)];
+		if (length(refFrame)>10)
+		{
+			refFrame <- as.data.frame(data[refFrame,]);
+		}
+		else
+		{
+			warning("Ref IDs not in data. All data will be used");
+			refFrame <- as.data.frame(data);
+		}
+	}
 
 	if (!is.null(refMean))
 	{
@@ -13,11 +27,13 @@ FRESAScale <- function(data,refFrame=NULL,method=c("Norm","Order","OrderLogit","
 	{
 #		print(class(refFrame));
 		usedFeatures <- colnames(refFrame);
+		usedFeatures <- usedFeatures[sapply(refFrame,is.numeric)];
 #		print(usedFeatures)
-		outs <- lapply(refFrame,table);
+		outs <- lapply(refFrame[,usedFeatures],table);
 		outl <- numeric(length(outs));
 		for (i in 1:length(outs)) outl[i] <- length(outs[[i]]);
 #		print(outl)
+
 		usedFeatures <- usedFeatures[outl > 5];
 #		print(usedFeatures);
 	}
@@ -28,6 +44,7 @@ FRESAScale <- function(data,refFrame=NULL,method=c("Norm","Order","OrderLogit","
 		datRefUses <-  as.data.frame(refFrame[,usedFeatures]);
 		colnames(datRefUses) <- usedFeatures;
 		rownames(datRefUses) <- rownames(refFrame);
+		srownames <- rownames(data)
 
 		switch(method,
 				Norm =
@@ -88,15 +105,23 @@ FRESAScale <- function(data,refFrame=NULL,method=c("Norm","Order","OrderLogit","
 				RankInv =
 				{
 					data <- rankInverseNormalDataFrame(usedFeatures,data,refFrame,strata); 			
+				},
+				LRankInv =
+				{
+					iqrsdratio = 0.5;
+					data <- rankInverseNormalDataFrame(usedFeatures,data,refFrame,strata);
+					data[,usedFeatures] <- 4.0*(1.0/(1.0+exp(-iqrsdratio*data[,usedFeatures])) - 0.5)/iqrsdratio;
 				}
 			)
+		data <- data[srownames,]
 		if (!is.null(refMean))
 		{	
 			names(refMean) <- usedFeatures;
 			names(refDisp) <- usedFeatures;
 		}
 	}
-
-	result <- list(scaledData=as.data.frame(data),refMean=refMean,refDisp=refDisp,strata=strata,method=method);
+	scaledData=as.data.frame(data);
+	attr(scaledData,"usedFeatures") <- usedFeatures;
+	result <- list(scaledData=scaledData,refMean=refMean,refDisp=refDisp,strata=strata,method=method,refFrame=refFrame);
 	return (result);
 }
