@@ -1,5 +1,5 @@
 featureAdjustment <-
-function(variableList,baseModel,strata=NA,data,referenceframe,type=c("LM","GLS","RLM","NZLM","SPLINE","MARS","LOESS"),pvalue=0.05,correlationGroup = "ID",...) 
+function(variableList,baseFormula,strata=NA,data,referenceframe,type=c("LM","GLS","RLM","NZLM","SPLINE","MARS","LOESS"),pvalue=0.05,correlationGroup = "ID",...) 
 {
 
 if (!requireNamespace("nlme", quietly = TRUE)) {
@@ -12,9 +12,16 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 	install.packages("mda", dependencies = TRUE)
 }
 	type <- match.arg(type);
-	## the reference frame will be used to predict a variable from the basemodel. At output the residuals are returned.
+	## the reference frame will be used to predict a variable from the baseFormula. At output the residuals are returned.
 	## strata is a numeric column varname in the data frame from 0 to S, where S is the maximum number of strata
-	colnamesList <- as.vector(variableList[,1]);
+	if (inherits(variableList,"character"))
+	{
+		colnamesList <- variableList;
+	}
+	else
+	{
+		colnamesList <- as.vector(variableList[,1]);
+	}
 	size = length(colnamesList);
 	if (!is.na(strata)) 
 	{
@@ -33,7 +40,7 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 	created = 0;
 	models <- list();
 	idx <- 1; 
-	tbbaseModel <- NULL;
+	tbbaseFormula <- NULL;
 	AdjustedFrame <- NULL;
 	isContinous <- FALSE;
 	datamodel <- NULL;
@@ -56,9 +63,9 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 		}
 		if ((nrow(cstrata)>1) && ( nrow(cstrataref)>1))
 		{
-			if (sum(str_count(baseModel,"\\+")) == 0)
+			if (sum(str_count(baseFormula,"\\+")) == 0)
 			{
-				datamodel <- cstrataref[,baseModel]
+				datamodel <- cstrataref[,baseFormula]
 				isContinous <- length(table(datamodel)) > 5;
 			}
 			for (i in 1:size)
@@ -69,7 +76,7 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 				{
 					avgref <- mean(dtacolumn,na.rm = TRUE);
 					ress1 <- dtacolumn - avgref;
-					ftm1 <- paste(colnamesList[i],paste(" ~ ",baseModel));
+					ftm1 <- paste(colnamesList[i],paste(" ~ ",baseFormula));
 					ftmp <- formula(ftm1);
 					mfref <- model.frame(ftmp,cstrataref);
 					mfstrata <- model.frame(ftmp,cstrata);
@@ -80,20 +87,20 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 					if (!inherits(modellm, "try-error"))
 					{	
 						ress2 <- modellm$residuals
-						plm <- .Call("improvedResidualsCpp",ress1,ress2,"Wilcox",0)$p.value
-						model <- modellm;
-						p <- plm;
+						p <- .Call("improvedResidualsCpp",ress1,ress2,"Wilcox",0)$p.value
+						plm <- p;
 						if (isContinous)
 						{
 							plm <- cor.test(datamodel,dtacolumn,method="spearman")$p.value
 						}
+						f <- summary(modellm)$fstatistic
+						pft <- pf(f[1],f[2],f[3],lower.tail=FALSE);
+						if (is.na(pft)) pft <- 1.0;
+						plm <- min(plm,pft)
+						p <- min(p,plm)
+						plm <- p
 					}
 					modelRLM <- modellm;
-#					f <- summary(modellm)$fstatistic
-#					pft <- pf(f[1],f[2],f[3],lower.tail=FALSE);
-#					if (is.na(pft)) pft <- 1.0;
-#					if (is.na(plm)) plm <- 1.0;
-#					plm <- min(plm,0.5*pft)
 					switch(type,
     					LOESS =
 						{
@@ -116,23 +123,6 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 									pred[is.na(pred)] <- predlm[is.na(pred)];
 									ress <- dtacolumn - pred;
 									p <- .Call("improvedResidualsCpp",ress1,ress,"Wilcox",0)$p.value
-#									sdd <- 3.0*median(abs(ress)) + 2.0*mean(abs(ress))
-#									if (sdd == 0)
-#									{
-#										sdd <- 1.0
-#									}									
-#									wts <- exp(-(ress/sdd)^2)
-#									dgf = length(ress)*min(model$pars$span,1.0)-model$pars$degree;
-#									rss1 <- sum(wts*ress1^2)
-#									rss2 <- sum(wts*ress^2)
-#									pft <- 0;
-#									if (rss2 > 0)
-#									{
-#										pft <- pf(dgf*rss1/rss2-dgf,1,dgf,lower.tail=FALSE)
-#									}
-#									if (is.na(p)) p <- 1.0;
-#									if (is.na(pft)) pft <- 1.0;
-#									p <- min(p,0.5*pft);
 								}								
 								else
 								{
@@ -155,23 +145,6 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 									ress <- dtacolumn - pred;
 									p <- .Call("improvedResidualsCpp",ress1,ress,"Wilcox",0)$p.value
 
-#									sdd <- 3.0*median(abs(ress)) + 2.0*mean(abs(ress))
-#									if (sdd == 0)
-#									{
-#										sdd <- 1.0
-#									}									
-#									wts <- exp(-(ress/sdd)^2)
-#									dgf = length(ress) - length(model$coefficients);
-#									rss1 <- sum(wts*ress1^2)
-#									rss2 <- sum(wts*ress^2)
-#									pft <- 0;
-#									if (rss2 > 0)
-#									{
-#										pft <- pf(dgf*rss1/rss2-dgf,1,dgf,lower.tail=FALSE)
-#									}
-#									if (is.na(p)) p <- 1.0;
-#									if (is.na(pft)) pft <- 1.0;
-#									p <- min(p,0.5*pft);
 								}								
 								else
 								{
@@ -195,23 +168,6 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 									ress <- dtacolumn - pred$y;
 									p <- .Call("improvedResidualsCpp",ress1,ress,"Wilcox",0)$p.value
 
-#									sdd <- 3.0*median(abs(ress)) + 2.0*mean(abs(ress))
-#									if (sdd == 0)
-#									{
-#										sdd <- 1.0
-#									}									
-#									wts <- exp(-(ress/sdd)^2)
-#									dgf = length(ress) - model$fit$nk;
-#									rss1 <- sum(wts*ress1^2)
-#									rss2 <- sum(wts*ress^2)
-#									pft <- 0;
-#									if (rss2 > 0)
-#									{
-#										pft <- pf(dgf*rss1/rss2-dgf,1,dgf,lower.tail=FALSE)
-#									}
-#									if (is.na(p)) p <- 1.0;
-#									if (is.na(pft)) pft <- 1.0;
-#									p <- min(p,0.5*pft);
 								}
 								else
 								{
@@ -235,21 +191,6 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 									{
 										p <- .Call("improvedResidualsCpp",ress1,ress,"Wilcox",0)$p.value
 
-#										model$w[is.na(model$w)] <- 1.0;
-#										model$w[model$w == 0] <- 1.0e-5;
-#										sw <- sum(model$w);
-#										dgf = length(ress)-length(model$coef)+1;
-#										m1 <- sum(model$w*dtacolumn,na.rm = TRUE)/sw
-#										rss1 <- sum(model$w*(dtacolumn^2),na.rm = TRUE)/sw-m1*m1
-#										rss2 <- sum(model$w*(ress^2),na.rm = TRUE)/sw
-#										pft <- 0;
-#										if (rss2 > 0)
-#										{
-#											pft <- pf(dgf*rss1/rss2-dgf,1,dgf,lower.tail=FALSE)
-#										}
-#										if (is.na(p)) p <- 1.0;
-#										if (is.na(pft)) pft <- 1.0;
-#										p <- min(p,0.5*pft);
 									}
 								}
 								else
@@ -297,9 +238,9 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 							{ 
 								if (p < pvalue)
 								{
-									if (class(model) == "mars")
+									if (inherits(model,"mars"))
 									{
-										cstrata[,colnamesList[i]] <- avgref + cstrata[,colnamesList[i]] - as.numeric(predict(model,cstrata[,baseModel]));
+										cstrata[,colnamesList[i]] <- avgref + cstrata[,colnamesList[i]] - as.numeric(predict(model,cstrata[,baseFormula]));
 									}
 									else
 									{
@@ -311,9 +252,9 @@ if (!requireNamespace("mda", quietly = TRUE)) {
 							{ 
 								if (p < pvalue)
 								{
-									if (class(model) == "smooth.spline")
+									if (inherits(model,"smooth.spline"))
 									{
-										cstrata[,colnamesList[i]] <-  avgref + cstrata[,colnamesList[i]] - predict(model,cstrata[,baseModel])$y;
+										cstrata[,colnamesList[i]] <-  avgref + cstrata[,colnamesList[i]] - predict(model,cstrata[,baseFormula])$y;
 									}
 									else
 									{
